@@ -17,7 +17,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '2.5.5';
+    $VERSION = '2.5.6';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -381,7 +381,7 @@ sub add_defaults {
             t                   => 1,
         },
         'scan-reload-triggers' => qw(mode),
-        message                => undef,
+        message                => {},
     );
     if (
         defined $SETTING->{frontend}
